
import java.util.HashMap;
import java.util.Map;

public class Theater {

	private final Map<Integer, Reservation> seats;
	private final int theaterSize;

	/**
	 * Create a new Theater of a given size.
	 */
	public Theater(int size) {
		seats = new HashMap<Integer, Reservation>();
		theaterSize = size;
	}

	/**
	 * Return the number of seats in this theater.
	 */
	public int getSize() {
		return theaterSize;
	}
	
	/**
	 * Make a reservation.
	 */
	public synchronized Reservation makeReservation(String personName, int howManySeats) {
		// see if we can find a block of seats
		int bestStart = 0;

		for (int x = 0; x < theaterSize; x++) {
			if (!seats.containsKey(x) && x == bestStart + howManySeats - 1) {
				// found contiguous block of seats
				break;
			} else if (seats.containsKey(x)) {
				bestStart = x + 1;
			}
		}

		if (bestStart <= theaterSize - howManySeats) {
			// found a block; make the reservation
			Reservation r = new Reservation(personName, bestStart, howManySeats);
			for (int x = bestStart; x < bestStart + howManySeats; x++) {
				seats.put(x, r);
			}
			return r;
		} else {
			// theater full
			return null;
		}
	}

	/**
	 * Return the Reservation for the seat number specified, or null
	 * if that seat is empty at the moment.
	 */
	public Reservation getReservationForSeat(int seatnum) {
		return seats.get(seatnum);
	}

	/**
	 * Return the Reservation for the given person's name, or null if
	 * there is no reservation under that name.
	 */
	public Reservation lookupReservation(String personName) {
		for (int i = 0; i < theaterSize; i++) {
			Reservation r = seats.get(i);
			if ((r != null) && (r.getName().equals(personName))) {
				return r;
			}
		}
		return null;
	}

	/**
	 * Cancel the Reservation for this person's name and re-open their
	 * seats.  If the person does not exist, this method does nothing.
	 */
	public synchronized boolean cancelReservation(String personName) {
		boolean retval = false;
		for (int i = 0; i < theaterSize; i++) {
			Reservation r = seats.get(i);
			if ((r != null) && (r.getName().equals(personName))) {
				seats.remove(i);
				retval = true;
			}
		}
		return retval;
	}

	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("[");
		for (int x = 0; x < theaterSize; x++) {
			if (seats.containsKey(x)) {
				sb.append("x");
			} else {
				sb.append("-");
			}

		}
		sb.append("]");
		return sb.toString();
	}
}
